@@ -6,31 +6,33 @@ import posixpath
 import shlex
 from collections import namedtuple
 from dataclasses import asdict
-from functools import partial
 
 import click
-from pygments import highlight, lexers, formatters
-from pymobiledevice3.cli.cli_common import print_json, Command
+from pykdebugparser.pykdebugparser import PyKdebugParser
+from pymobiledevice3.cli.cli_common import print_json, Command, default_json_encoder
 from pymobiledevice3.exceptions import DvtDirListError
 from pymobiledevice3.lockdown import LockdownClient
+from pymobiledevice3.services.debugserver_applist import DebugServerAppList
 from pymobiledevice3.services.dvt.dvt_secure_socket_proxy import DvtSecureSocketProxyService
 from pymobiledevice3.services.dvt.instruments.activity_trace_tap import ActivityTraceTap, decode_message_format
 from pymobiledevice3.services.dvt.instruments.application_listing import ApplicationListing
 from pymobiledevice3.services.dvt.instruments.condition_inducer import ConditionInducer
-from pymobiledevice3.services.dvt.instruments.core_profile_session_tap import CoreProfileSessionTap, DgbFuncQual, \
-    ProcessData
+from pymobiledevice3.services.dvt.instruments.core_profile_session_tap import CoreProfileSessionTap
 from pymobiledevice3.services.dvt.instruments.device_info import DeviceInfo
 from pymobiledevice3.services.dvt.instruments.energy_monitor import EnergyMonitor
-from pymobiledevice3.services.dvt.instruments.kdebug_events_parser import KdebugEventsParser
+from pymobiledevice3.services.dvt.instruments.graphics import Graphics
 from pymobiledevice3.services.dvt.instruments.network_monitor import NetworkMonitor, ConnectionDetectionEvent
+from pymobiledevice3.services.dvt.instruments.notifications import Notifications
 from pymobiledevice3.services.dvt.instruments.process_control import ProcessControl
 from pymobiledevice3.services.dvt.instruments.sysmontap import Sysmontap
 from pymobiledevice3.services.accessibilityaudit import AccessibilityAudit
 from pymobiledevice3.services.os_trace import OsTraceService
+from pymobiledevice3.services.remote_server import RemoteServer
 from pymobiledevice3.services.screenshot import ScreenshotService
 from pymobiledevice3.services.dtfetchsymbols import DtFetchSymbols
 from pymobiledevice3.services.simulate_location import DtSimulateLocation
 from termcolor import colored
+from pymobiledevice3.tcp_forwarder import TcpForwarder
 
 
 def wait_return():
@@ -56,16 +58,24 @@ def developer():
     pass
 
 
-@developer.command(cls=Command)
-@click.argument('out', type=click.File('wb'))
-def screenshot(lockdown, out):
-    """ take a screenshot in PNG format """
-    out.write(ScreenshotService(lockdown=lockdown).take_screenshot())
+@developer.command('shell', cls=Command)
+@click.argument('service')
+@click.option('-r', '--remove-ssl-context', is_flag=True)
+def developer_shell(lockdown, service, remove_ssl_context):
+    """ Launch developer shell. """
+    with RemoteServer(lockdown, service, remove_ssl_context) as service:
+        service.shell()
 
 
-@developer.command('proclist', cls=Command)
-@click.option('--nocolor', is_flag=True)
-def proclist(lockdown, nocolor):
+@developer.group()
+def dvt():
+    """ dvt operations """
+    pass
+
+
+@dvt.command('proclist', cls=Command)
+@click.option('--color/--no-color', default=True)
+def proclist(lockdown, color):
     """ show process list """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         processes = DeviceInfo(dvt).proclist()
@@ -73,19 +83,19 @@ def proclist(lockdown, nocolor):
             if 'startDate' in process:
                 process['startDate'] = str(process['startDate'])
 
-        print_json(processes, colored=not nocolor)
+        print_json(processes, colored=color)
 
 
-@developer.command('applist', cls=Command)
-@click.option('--nocolor', is_flag=True)
-def applist(lockdown, nocolor):
+@dvt.command('applist', cls=Command)
+@click.option('--color/--no-color', default=True)
+def applist(lockdown, color):
     """ show application list """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         apps = ApplicationListing(dvt).applist()
-        print_json(apps, colored=not nocolor)
+        print_json(apps, colored=color)
 
 
-@developer.command('kill', cls=Command)
+@dvt.command('kill', cls=Command)
 @click.argument('pid', type=click.INT)
 def kill(lockdown, pid):
     """ Kill a process by its pid. """
@@ -93,7 +103,7 @@ def kill(lockdown, pid):
         ProcessControl(dvt).kill(pid)
 
 
-@developer.command('pkill', cls=Command)
+@dvt.command('pkill', cls=Command)
 @click.argument('expression')
 def pkill(lockdown, expression):
     """ kill all processes containing `expression` in their name. """
@@ -112,7 +122,7 @@ def pkill(lockdown, expression):
                 process_control.kill(pid)
 
 
-@developer.command('launch', cls=Command)
+@dvt.command('launch', cls=Command)
 @click.argument('arguments', type=click.STRING)
 @click.option('--kill-existing/--no-kill-existing', default=True)
 @click.option('--suspended', is_flag=True)
@@ -130,8 +140,8 @@ def launch(lockdown: LockdownClient, arguments: str, kill_existing: bool, suspen
         print(f'Process launched with pid {pid}')
 
 
-@developer.command('shell', cls=Command)
-def shell(lockdown):
+@dvt.command('shell', cls=Command)
+def dvt_shell(lockdown):
     """ Launch developer shell. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         dvt.shell()
@@ -150,7 +160,7 @@ def show_dirlist(device_info: DeviceInfo, dirname, recursive=False):
             show_dirlist(device_info, filename, recursive=recursive)
 
 
-@developer.command('ls', cls=Command)
+@dvt.command('ls', cls=Command)
 @click.argument('path', type=click.Path(exists=False))
 @click.option('-r', '--recursive', is_flag=True)
 def ls(lockdown, path, recursive):
@@ -159,9 +169,9 @@ def ls(lockdown, path, recursive):
         show_dirlist(DeviceInfo(dvt), path, recursive=recursive)
 
 
-@developer.command('device-information', cls=Command)
-@click.option('--nocolor', is_flag=True)
-def device_information(lockdown, nocolor):
+@dvt.command('device-information', cls=Command)
+@click.option('--color/--no-color', default=True)
+def device_information(lockdown, color):
     """ Print system information. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         device_info = DeviceInfo(dvt)
@@ -169,10 +179,10 @@ def device_information(lockdown, nocolor):
             'system': device_info.system_information(),
             'hardware': device_info.hardware_information(),
             'network': device_info.network_information(),
-        }, colored=not nocolor)
+        }, colored=color)
 
 
-@developer.command('netstat', cls=Command)
+@dvt.command('netstat', cls=Command)
 def netstat(lockdown):
     """ Print information about current network activity. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
@@ -184,7 +194,7 @@ def netstat(lockdown):
                         f'{event.remote_address.data.address}:{event.remote_address.port}')
 
 
-@developer.group('sysmon')
+@dvt.group('sysmon')
 def sysmon():
     """ System monitor options. """
 
@@ -281,7 +291,7 @@ def sysmon_system(lockdown, fields):
             print(f'{name}: {value}')
 
 
-@developer.group('core-profile-session')
+@dvt.group('core-profile-session')
 def core_profile_session():
     """ Core profile session options. """
 
@@ -300,11 +310,6 @@ def parse_filters(filters):
     return parsed
 
 
-def format_timestamp(core_session_tap, timestamp):
-    time_string = core_session_tap.parse_event_time(timestamp).strftime('%Y-%m-%d %H:%M:%S.%f')
-    return f'{time_string:<27}'
-
-
 @core_profile_session.command('live', cls=Command)
 @click.option('-c', '--count', type=click.INT, default=-1, help='Number of events to print. Omit to endless sniff.')
 @click.option('-f', '--filters', multiple=True, help='Events filter. Omit for all.')
@@ -320,42 +325,29 @@ def live_profile_session(lockdown, count, filters, pid, tid, timestamp, event_na
                          show_tid, process_name, args):
     """ Print core profiling information. """
     filters = parse_filters(filters)
+    parser = PyKdebugParser()
+    parser.filter_tid = tid
+    parser.show_timestamp = timestamp
+    parser.show_name = event_name
+    parser.show_func_qual = func_qual
+    parser.show_tid = show_tid
+    parser.show_process = process_name
+    parser.show_args = args
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         trace_codes_map = DeviceInfo(dvt).trace_codes()
-        print('Receiving time information')
         time_config = CoreProfileSessionTap.get_time_config(dvt)
+        parser.numer = time_config['numer']
+        parser.denom = time_config['denom']
+        parser.mach_absolute_time = time_config['mach_absolute_time']
+        parser.usecs_since_epoch = time_config['usecs_since_epoch']
+        parser.timezone = time_config['timezone']
         with CoreProfileSessionTap(dvt, time_config, filters) as tap:
-            for event in tap.watch_events(count):
-                if event.eventid in trace_codes_map:
-                    name = trace_codes_map[event.eventid] + f' ({hex(event.eventid)})'
-                else:
-                    # Some event IDs are not public.
-                    name = hex(event.eventid)
-                if tid is not None and event.tid != tid:
-                    continue
-                try:
-                    process = tap.thread_map[event.tid]
-                except KeyError:
-                    process = ProcessData(pid=-1, name='')
-                if pid is not None and process.pid != pid:
-                    continue
-                formatted_data = ''
-                if timestamp:
-                    formatted_data += format_timestamp(tap, event.timestamp)
-                formatted_data += f'{name:<58}' if event_name else ''
-                if func_qual:
-                    try:
-                        formatted_data += f'{DgbFuncQual(event.func_qualifier).name:<15}'
-                    except ValueError:
-                        formatted_data += f'''{'Error':<16}'''
-                formatted_data += f'{hex(event.tid):<12}' if show_tid else ''
-                if process_name:
-                    process_rep = (f'{process.name}({process.pid})'
-                                   if process.pid != -1
-                                   else f'Error: tid {event.tid}')
-                    formatted_data += f'{process_rep:<27}' if process_name else ''
-                formatted_data += f'{str(event.data):<34}' if args else ''
-                print(formatted_data)
+            i = 0
+            for event in parser.formatted_kevents(tap.get_kdbuf_stream(), trace_codes_map):
+                print(event)
+                i += 1
+                if i == count:
+                    break
 
 
 @core_profile_session.command('save', cls=Command)
@@ -371,42 +363,16 @@ def save_profile_session(lockdown, out, filters):
 
 @core_profile_session.command('stackshot', cls=Command)
 @click.option('--out', type=click.File('w'), default=None)
-@click.option('--nocolor', is_flag=True)
-def stackshot(lockdown, out, nocolor):
+@click.option('--color/--no-color', default=True)
+def stackshot(lockdown, out, color):
     """ Dump stackshot information. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         with CoreProfileSessionTap(dvt, {}) as tap:
             data = tap.get_stackshot()
             if out is not None:
-                json.dump(data, out, indent=4)
+                json.dump(data, out, indent=4, default=default_json_encoder)
             else:
-                print_json(data, colored=not nocolor)
-
-
-def parse_live_print(tap, pid, show_tid, parsed, nocolor):
-    tid = parsed.ktraces[0].tid
-    try:
-        process = tap.thread_map[tid]
-    except KeyError:
-        process = ProcessData(pid=-1, name='')
-    if pid is not None and process.pid != pid:
-        return
-    if parsed is None:
-        return
-
-    formatted_data = format_timestamp(tap, parsed.ktraces[0].timestamp)
-    formatted_data += f'{tid:>11} ' if show_tid else ''
-    process_rep = (f'{process.name}({process.pid})'
-                   if process.pid != -1
-                   else f'Error: tid {tid}')
-    formatted_data += f'{process_rep:<34}'
-    if nocolor:
-        event_rep = str(parsed)
-    else:
-        event_rep = highlight(
-            str(parsed), lexers.CLexer(), formatters.TerminalTrueColorFormatter(style='stata-dark')).strip()
-
-    print(formatted_data + event_rep)
+                print_json(data, colored=color)
 
 
 @core_profile_session.command('parse-live', cls=Command)
@@ -415,8 +381,8 @@ def parse_live_print(tap, pid, show_tid, parsed, nocolor):
 @click.option('--tid', type=click.INT, default=None, help='Thread ID to filter. Omit for all.')
 @click.option('--show-tid/--no-show-tid', default=False, help='Whether to print thread id or not.')
 @click.option('-f', '--filters', multiple=True, help='Events filter. Omit for all.')
-@click.option('--nocolor', is_flag=True, help='disable colors')
-def parse_live_profile_session(lockdown, count, pid, tid, show_tid, filters, nocolor):
+@click.option('--color/--no-color', default=True)
+def parse_live_profile_session(lockdown, count, pid, tid, show_tid, filters, color):
     """ Parse core profiling information. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         trace_codes_map = DeviceInfo(dvt).trace_codes()
@@ -424,32 +390,42 @@ def parse_live_profile_session(lockdown, count, pid, tid, show_tid, filters, noc
         filters = parse_filters(filters)
         print('Receiving time information')
         time_config = CoreProfileSessionTap.get_time_config(dvt)
+        parser = PyKdebugParser()
+        parser.numer = time_config['numer']
+        parser.denom = time_config['denom']
+        parser.mach_absolute_time = time_config['mach_absolute_time']
+        parser.usecs_since_epoch = time_config['usecs_since_epoch']
+        parser.timezone = time_config['timezone']
+        parser.filter_tid = tid
+        parser.color = color
+
         with CoreProfileSessionTap(dvt, time_config, filters) as tap:
-            events_callback = partial(parse_live_print, tap, pid, show_tid, nocolor=nocolor)
-            events_parser = KdebugEventsParser(events_callback, trace_codes_map, tap.thread_map)
             if show_tid:
-                print('{:^26}|{:^11}|{:^33}|   Event'.format('Time', 'Thread', 'Process'))
+                print('{:^32}|{:^11}|{:^33}|   Event'.format('Time', 'Thread', 'Process'))
             else:
-                print('{:^26}|{:^33}|   Event'.format('Time', 'Process'))
-            for event in tap.watch_events(count):
-                if tid is not None and event.tid != tid:
-                    continue
-                events_parser.feed(event)
+                print('{:^32}|{:^33}|   Event'.format('Time', 'Process'))
+
+            i = 0
+            for trace in parser.formatted_traces(tap.get_kdbuf_stream(), trace_codes_map):
+                print(trace)
+                i += 1
+                if i == count:
+                    break
 
 
-@developer.command('trace-codes', cls=Command)
-@click.option('--nocolor', is_flag=True)
-def trace_codes(lockdown, nocolor):
+@dvt.command('trace-codes', cls=Command)
+@click.option('--color/--no-color', default=True)
+def trace_codes(lockdown, color):
     """ Print system information. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         device_info = DeviceInfo(dvt)
-        print_json({hex(k): v for k, v in device_info.trace_codes().items()}, colored=not nocolor)
+        print_json({hex(k): v for k, v in device_info.trace_codes().items()}, colored=color)
 
 
-@developer.command('oslog', cls=Command)
-@click.option('--nocolor', is_flag=True, help='disable colors')
+@dvt.command('oslog', cls=Command)
+@click.option('--color/--no-color', default=True)
 @click.option('--pid', type=click.INT)
-def developer_oslog(lockdown, nocolor, pid):
+def dvt_oslog(lockdown, color, pid):
     """ oslog. """
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         with ActivityTraceTap(dvt) as tap:
@@ -470,7 +446,7 @@ def developer_oslog(lockdown, nocolor, pid):
                 except Exception:
                     print('error decoding')
 
-                if not nocolor:
+                if color:
                     message_pid = colored(str(message_pid), 'magenta')
                     subsystem = colored(subsystem, 'green')
                     category = colored(category, 'green')
@@ -480,9 +456,9 @@ def developer_oslog(lockdown, nocolor, pid):
                 print(f'[{subsystem}][{category}][{message_pid}][{image_name}] <{message_type}>: {formatted_message}')
 
 
-@developer.command('energy', cls=Command)
+@dvt.command('energy', cls=Command)
 @click.argument('pid-list', nargs=-1)
-def developer_energy(lockdown, pid_list):
+def dvt_energy(lockdown, pid_list):
     """ energy monitoring for given pid list. """
 
     if len(pid_list) == 0:
@@ -497,6 +473,24 @@ def developer_energy(lockdown, pid_list):
                 logging.info(telemetry)
 
 
+@dvt.command('notifications', cls=Command)
+def dvt_notifications(lockdown):
+    """ monitor memory and app notifications """
+    with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
+        with Notifications(dvt) as notifications:
+            for notification in notifications:
+                logging.info(notification)
+
+
+@dvt.command('graphics', cls=Command)
+def dvt_notifications(lockdown):
+    """ monitor graphics statistics """
+    with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
+        with Graphics(dvt) as graphics:
+            for stats in graphics:
+                logging.info(stats)
+
+
 @developer.command('fetch-symbols', cls=Command)
 @click.argument('out', type=click.Path(dir_okay=True, file_okay=False))
 def developer_fetch_symbols(lockdown, out):
@@ -639,3 +633,39 @@ def condition_set(lockdown, profile_identifier):
     with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
         ConditionInducer(dvt).set(profile_identifier)
         wait_return()
+
+
+@developer.command(cls=Command)
+@click.argument('out', type=click.File('wb'))
+def screenshot(lockdown, out):
+    """ take a screenshot in PNG format """
+    out.write(ScreenshotService(lockdown=lockdown).take_screenshot())
+
+
+@developer.group('debugserver')
+def debugserver():
+    """ debugserver options. """
+    pass
+
+
+@debugserver.command('applist', cls=Command)
+def debugserver_applist(lockdown):
+    """ get applist xml """
+    print_json(DebugServerAppList(lockdown).get())
+
+
+@debugserver.command('start-server', cls=Command)
+@click.argument('local_port', type=click.INT)
+def debugserver_shell(lockdown, local_port):
+    """
+    start a debugserver at remote listening on a given port locally.
+
+    Please note the connection must be done soon afterwards using your own lldb client.
+    This can be done using the following commands within lldb shell:
+
+    - platform select remote-ios
+
+    - platform connect connect://localhost:<local_port>
+    """
+    attr = lockdown.get_service_connection_attributes('com.apple.debugserver.DVTSecureSocketProxy')
+    TcpForwarder(lockdown, local_port, attr['Port'], attr.get('EnableServiceSSL', False)).start()
