@@ -15,16 +15,3 @@ def test_stackshot(lockdown):
     # Constant kernel task data.
     assert data['task_snapshots'][0]['task_snapshot']['ts_pid'] == 0
     assert data['task_snapshots'][0]['task_snapshot']['ts_p_comm'] == 'kernel_task'
-
-
-def test_watch_events(lockdown):
-    """
-    Test getting realtime kdebug events.
-    :param pymobiledevice3.lockdown.LockdownClient lockdown: Lockdown client.
-    """
-    events_count = 10
-    with DvtSecureSocketProxyService(lockdown=lockdown) as dvt:
-        with CoreProfileSessionTap(dvt, CoreProfileSessionTap.get_time_config(dvt)) as tap:
-            events = list(tap.watch_events(events_count))
-
-    assert len(events) == events_count
